pub struct Cursor<'i, T> { /* private fields */ }Expand description
A copy of a cursor of an IterList.
Allows for traversing the list without modifying the original.
Internally, the cursor is a fat pointer to the current element,
so the size of a Cursor is two words.
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
assert_eq!(cursor.next(), Some(&1));
assert_eq!(cursor.next(), Some(&2));
assert_eq!(cursor.current(), Some(&3));
assert_eq!(list.current(), Some(&1));Implementations
Sourceimpl<'t, T> Cursor<'t, T>
impl<'t, T> Cursor<'t, T>
Sourcepub const unsafe fn from_raw(ptr: *mut u8) -> Self
pub const unsafe fn from_raw(ptr: *mut u8) -> Self
Create a new cursor from a raw pointer. O(1).
Safety
The pointer must be valid and point to a Node in a valid IterList.
Sourcepub const unsafe fn new_dangling() -> Self
pub const unsafe fn new_dangling() -> Self
Create a new dangling cursor. O(1).
The cursor will be invalid until Cursor::reacquire is called on a non-empty list.
Safety
Calling most methods on a dangling cursor will result in UB.
Sourcepub const fn new() -> Self
pub const fn new() -> Self
Creates an empty cursor. O(1).
The cursor will be invalid until Cursor::reacquire is called on a non-empty list.
Sourcepub fn from(list: &'t IterList<T>) -> Self
pub fn from(list: &'t IterList<T>) -> Self
Create a new cursor from an IterList. O(1).
The new cursor is an exact copy of the list’s cursor.
Sourcepub fn reacquire(&mut self, list: &'t IterList<T>)
pub fn reacquire(&mut self, list: &'t IterList<T>)
Update the cursor to match the current state of the list. O(1).
Useful if you lose track of the list, or want to use the same cursor on multiple lists.
Sourcepub fn current(&self) -> Option<&T>
pub fn current(&self) -> Option<&T>
Get a ref to the current element. O(1).
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
assert_eq!(cursor.current(), Some(&1));Sourcepub const fn index(&self) -> usize
pub const fn index(&self) -> usize
Get the index of the cursor O(1).
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
cursor.next();
assert_eq!(cursor.index(), 1);Sourcepub fn move_to_front(&mut self) -> usize
pub fn move_to_front(&mut self) -> usize
Move the cursor to the front of the list. O(n).
Returns the number of elements traversed.
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
cursor.move_to_back();
assert_eq!(cursor.current(), Some(&3));
let offset = cursor.move_to_front();
assert_eq!(offset, 2);
assert_eq!(cursor.current(), Some(&1));Sourcepub fn move_to_back(&mut self) -> usize
pub fn move_to_back(&mut self) -> usize
Move the cursor to the back of the list. O(n).
Returns the number of elements traversed.
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
let offset = cursor.move_to_back();
assert_eq!(offset, 2);
assert_eq!(cursor.current(), Some(&3));Sourcepub fn move_to(&mut self, index: usize) -> bool
pub fn move_to(&mut self, index: usize) -> bool
Move the cursor to the specified index. O(n).
If the index is out of bounds the cursor will be moved to the edge,
and false will be returned.
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
cursor.move_to(1);
assert_eq!(cursor.current(), Some(&2));Panics
Panics if the index is out of bounds.
Sourcepub fn advance(&mut self) -> bool
pub fn advance(&mut self) -> bool
Move the cursor one step forward. O(1).
Returns false if the cursor could not be moved.
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
assert_eq!(cursor.current(), Some(&1));
cursor.advance();
assert_eq!(cursor.current(), Some(&2));Sourcepub fn retreat(&mut self) -> bool
pub fn retreat(&mut self) -> bool
Move the cursor one step backward. O(1).
Returns false if the cursor could not be moved.
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
cursor.move_to_back();
assert_eq!(cursor.current(), Some(&3));
cursor.retreat();
assert_eq!(cursor.current(), Some(&2));Sourcepub fn move_by(&mut self, offset: isize) -> bool
pub fn move_by(&mut self, offset: isize) -> bool
Move the cursor by a given offset. O(n).
If the offset is out of bounds the cursor will be moved to the edge,
and false will be returned.
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
cursor.move_to_back();
assert_eq!(cursor.index(), 2);
cursor.move_by(-2);
assert_eq!(cursor.index(), 0);
cursor.move_by(10);
assert_eq!(cursor.index(), 2);Panics
Panics if the offset is out of bounds.
Sourcepub fn get(&self, offset: isize) -> Option<&T>
pub fn get(&self, offset: isize) -> Option<&T>
Get a ref to an element at the given offset. O(n).
Returns None if the offset is out of bounds.
let list = IterList::from(vec![1, 2, 3]);
let mut cursor = list.as_cursor();
assert_eq!(cursor.current(), Some(&1));
assert_eq!(cursor.get(1), Some(&2));
assert_eq!(cursor.get(-1), None);Trait Implementations
Sourceimpl<'i, T> Iterator for Cursor<'i, T>
impl<'i, T> Iterator for Cursor<'i, T>
Sourcefn next(&mut self) -> Option<Self::Item>
fn next(&mut self) -> Option<Self::Item>
Sourcefn next_chunk<const N: usize>(
&mut self,
) -> Result<[Self::Item; N], IntoIter<Self::Item, N>>where
Self: Sized,
fn next_chunk<const N: usize>(
&mut self,
) -> Result<[Self::Item; N], IntoIter<Self::Item, N>>where
Self: Sized,
iter_next_chunk)N values. Read more1.0.0 (const: unstable) · Sourcefn size_hint(&self) -> (usize, Option<usize>)
fn size_hint(&self) -> (usize, Option<usize>)
1.0.0 (const: unstable) · Sourcefn count(self) -> usizewhere
Self: Sized,
fn count(self) -> usizewhere
Self: Sized,
1.0.0 (const: unstable) · Sourcefn last(self) -> Option<Self::Item>where
Self: Sized,
fn last(self) -> Option<Self::Item>where
Self: Sized,
Sourcefn advance_by(&mut self, n: usize) -> Result<(), NonZero<usize>>
fn advance_by(&mut self, n: usize) -> Result<(), NonZero<usize>>
iter_advance_by)n elements. Read more1.0.0 (const: unstable) · Sourcefn nth(&mut self, n: usize) -> Option<Self::Item>
fn nth(&mut self, n: usize) -> Option<Self::Item>
nth element of the iterator. Read more1.28.0 (const: unstable) · Sourcefn step_by(self, step: usize) -> StepBy<Self>where
Self: Sized,
fn step_by(self, step: usize) -> StepBy<Self>where
Self: Sized,
1.0.0 (const: unstable) · Sourcefn chain<U>(self, other: U) -> Chain<Self, <U as IntoIterator>::IntoIter>
fn chain<U>(self, other: U) -> Chain<Self, <U as IntoIterator>::IntoIter>
1.0.0 (const: unstable) · Sourcefn zip<U>(self, other: U) -> Zip<Self, <U as IntoIterator>::IntoIter>where
Self: Sized,
U: IntoIterator,
fn zip<U>(self, other: U) -> Zip<Self, <U as IntoIterator>::IntoIter>where
Self: Sized,
U: IntoIterator,
Sourcefn intersperse(self, separator: Self::Item) -> Intersperse<Self>
fn intersperse(self, separator: Self::Item) -> Intersperse<Self>
iter_intersperse)separator between items
of the original iterator. Read moreSourcefn intersperse_with<G>(self, separator: G) -> IntersperseWith<Self, G>
fn intersperse_with<G>(self, separator: G) -> IntersperseWith<Self, G>
iter_intersperse)separator
between items of the original iterator. Read more1.0.0 (const: unstable) · Sourcefn map<B, F>(self, f: F) -> Map<Self, F>
fn map<B, F>(self, f: F) -> Map<Self, F>
1.21.0 (const: unstable) · Sourcefn for_each<F>(self, f: F)
fn for_each<F>(self, f: F)
1.0.0 (const: unstable) · Sourcefn filter<P>(self, predicate: P) -> Filter<Self, P>
fn filter<P>(self, predicate: P) -> Filter<Self, P>
1.0.0 (const: unstable) · Sourcefn filter_map<B, F>(self, f: F) -> FilterMap<Self, F>
fn filter_map<B, F>(self, f: F) -> FilterMap<Self, F>
1.0.0 (const: unstable) · Sourcefn enumerate(self) -> Enumerate<Self>where
Self: Sized,
fn enumerate(self) -> Enumerate<Self>where
Self: Sized,
1.0.0 (const: unstable) · Sourcefn skip_while<P>(self, predicate: P) -> SkipWhile<Self, P>
fn skip_while<P>(self, predicate: P) -> SkipWhile<Self, P>
1.0.0 (const: unstable) · Sourcefn take_while<P>(self, predicate: P) -> TakeWhile<Self, P>
fn take_while<P>(self, predicate: P) -> TakeWhile<Self, P>
1.57.0 (const: unstable) · Sourcefn map_while<B, P>(self, predicate: P) -> MapWhile<Self, P>
fn map_while<B, P>(self, predicate: P) -> MapWhile<Self, P>
1.0.0 (const: unstable) · Sourcefn skip(self, n: usize) -> Skip<Self>where
Self: Sized,
fn skip(self, n: usize) -> Skip<Self>where
Self: Sized,
n elements. Read more1.0.0 (const: unstable) · Sourcefn take(self, n: usize) -> Take<Self>where
Self: Sized,
fn take(self, n: usize) -> Take<Self>where
Self: Sized,
n elements, or fewer
if the underlying iterator ends sooner. Read more1.0.0 (const: unstable) · Sourcefn scan<St, B, F>(self, initial_state: St, f: F) -> Scan<Self, St, F>
fn scan<St, B, F>(self, initial_state: St, f: F) -> Scan<Self, St, F>
1.0.0 (const: unstable) · Sourcefn flat_map<U, F>(self, f: F) -> FlatMap<Self, U, F>
fn flat_map<U, F>(self, f: F) -> FlatMap<Self, U, F>
1.29.0 (const: unstable) · Sourcefn flatten(self) -> Flatten<Self>
fn flatten(self) -> Flatten<Self>
Sourcefn map_windows<F, R, const N: usize>(self, f: F) -> MapWindows<Self, F, N>
fn map_windows<F, R, const N: usize>(self, f: F) -> MapWindows<Self, F, N>
iter_map_windows)f for each contiguous window of size N over
self and returns an iterator over the outputs of f. Like slice::windows(),
the windows during mapping overlap as well. Read more1.0.0 (const: unstable) · Sourcefn inspect<F>(self, f: F) -> Inspect<Self, F>
fn inspect<F>(self, f: F) -> Inspect<Self, F>
1.0.0 (const: unstable) · Sourcefn by_ref(&mut self) -> &mut Selfwhere
Self: Sized,
fn by_ref(&mut self) -> &mut Selfwhere
Self: Sized,
Iterator. Read more1.0.0 (const: unstable) · Sourcefn collect<B>(self) -> B
fn collect<B>(self) -> B
Sourcefn try_collect<B>(
&mut self,
) -> <<Self::Item as Try>::Residual as Residual<B>>::TryType
fn try_collect<B>( &mut self, ) -> <<Self::Item as Try>::Residual as Residual<B>>::TryType
iterator_try_collect)Sourcefn collect_into<E>(self, collection: &mut E) -> &mut E
fn collect_into<E>(self, collection: &mut E) -> &mut E
iter_collect_into)1.0.0 (const: unstable) · Sourcefn partition<B, F>(self, f: F) -> (B, B)
fn partition<B, F>(self, f: F) -> (B, B)
Sourcefn is_partitioned<P>(self, predicate: P) -> bool
fn is_partitioned<P>(self, predicate: P) -> bool
iter_is_partitioned)true precede all those that return false. Read more1.27.0 (const: unstable) · Sourcefn try_fold<B, F, R>(&mut self, init: B, f: F) -> R
fn try_fold<B, F, R>(&mut self, init: B, f: F) -> R
1.27.0 (const: unstable) · Sourcefn try_for_each<F, R>(&mut self, f: F) -> R
fn try_for_each<F, R>(&mut self, f: F) -> R
1.0.0 (const: unstable) · Sourcefn fold<B, F>(self, init: B, f: F) -> B
fn fold<B, F>(self, init: B, f: F) -> B
1.51.0 (const: unstable) · Sourcefn reduce<F>(self, f: F) -> Option<Self::Item>
fn reduce<F>(self, f: F) -> Option<Self::Item>
Sourcefn try_reduce<R>(
&mut self,
f: impl FnMut(Self::Item, Self::Item) -> R,
) -> <<R as Try>::Residual as Residual<Option<<R as Try>::Output>>>::TryType
fn try_reduce<R>( &mut self, f: impl FnMut(Self::Item, Self::Item) -> R, ) -> <<R as Try>::Residual as Residual<Option<<R as Try>::Output>>>::TryType
iterator_try_reduce)1.0.0 (const: unstable) · Sourcefn all<F>(&mut self, f: F) -> bool
fn all<F>(&mut self, f: F) -> bool
1.0.0 (const: unstable) · Sourcefn any<F>(&mut self, f: F) -> bool
fn any<F>(&mut self, f: F) -> bool
1.0.0 (const: unstable) · Sourcefn find<P>(&mut self, predicate: P) -> Option<Self::Item>
fn find<P>(&mut self, predicate: P) -> Option<Self::Item>
1.30.0 (const: unstable) · Sourcefn find_map<B, F>(&mut self, f: F) -> Option<B>
fn find_map<B, F>(&mut self, f: F) -> Option<B>
Sourcefn try_find<R>(
&mut self,
f: impl FnMut(&Self::Item) -> R,
) -> <<R as Try>::Residual as Residual<Option<Self::Item>>>::TryType
fn try_find<R>( &mut self, f: impl FnMut(&Self::Item) -> R, ) -> <<R as Try>::Residual as Residual<Option<Self::Item>>>::TryType
try_find)1.0.0 (const: unstable) · Sourcefn position<P>(&mut self, predicate: P) -> Option<usize>
fn position<P>(&mut self, predicate: P) -> Option<usize>
1.0.0 (const: unstable) · Sourcefn max(self) -> Option<Self::Item>
fn max(self) -> Option<Self::Item>
1.0.0 (const: unstable) · Sourcefn min(self) -> Option<Self::Item>
fn min(self) -> Option<Self::Item>
1.6.0 (const: unstable) · Sourcefn max_by_key<B, F>(self, f: F) -> Option<Self::Item>
fn max_by_key<B, F>(self, f: F) -> Option<Self::Item>
1.15.0 (const: unstable) · Sourcefn max_by<F>(self, compare: F) -> Option<Self::Item>
fn max_by<F>(self, compare: F) -> Option<Self::Item>
1.6.0 (const: unstable) · Sourcefn min_by_key<B, F>(self, f: F) -> Option<Self::Item>
fn min_by_key<B, F>(self, f: F) -> Option<Self::Item>
1.15.0 (const: unstable) · Sourcefn min_by<F>(self, compare: F) -> Option<Self::Item>
fn min_by<F>(self, compare: F) -> Option<Self::Item>
1.0.0 (const: unstable) · Sourcefn unzip<A, B, FromA, FromB>(self) -> (FromA, FromB)
fn unzip<A, B, FromA, FromB>(self) -> (FromA, FromB)
1.36.0 (const: unstable) · Sourcefn copied<'a, T>(self) -> Copied<Self>
fn copied<'a, T>(self) -> Copied<Self>
1.0.0 (const: unstable) · Sourcefn cycle(self) -> Cycle<Self>
fn cycle(self) -> Cycle<Self>
Sourcefn array_chunks<const N: usize>(self) -> ArrayChunks<Self, N>where
Self: Sized,
fn array_chunks<const N: usize>(self) -> ArrayChunks<Self, N>where
Self: Sized,
iter_array_chunks)N elements of the iterator at a time. Read more1.11.0 (const: unstable) · Sourcefn product<P>(self) -> P
fn product<P>(self) -> P
Sourcefn cmp_by<I, F>(self, other: I, cmp: F) -> Ordering
fn cmp_by<I, F>(self, other: I, cmp: F) -> Ordering
iter_order_by)Iterator with those
of another with respect to the specified comparison function. Read more1.5.0 (const: unstable) · Sourcefn partial_cmp<I>(self, other: I) -> Option<Ordering>
fn partial_cmp<I>(self, other: I) -> Option<Ordering>
PartialOrd elements of
this Iterator with those of another. The comparison works like short-circuit
evaluation, returning a result without comparing the remaining elements.
As soon as an order can be determined, the evaluation stops and a result is returned. Read moreSourcefn partial_cmp_by<I, F>(self, other: I, partial_cmp: F) -> Option<Ordering>where
Self: Sized,
I: IntoIterator,
F: FnMut(Self::Item, <I as IntoIterator>::Item) -> Option<Ordering>,
fn partial_cmp_by<I, F>(self, other: I, partial_cmp: F) -> Option<Ordering>where
Self: Sized,
I: IntoIterator,
F: FnMut(Self::Item, <I as IntoIterator>::Item) -> Option<Ordering>,
iter_order_by)Iterator with those
of another with respect to the specified comparison function. Read moreSourcefn eq_by<I, F>(self, other: I, eq: F) -> bool
fn eq_by<I, F>(self, other: I, eq: F) -> bool
iter_order_by)1.5.0 (const: unstable) · Sourcefn lt<I>(self, other: I) -> bool
fn lt<I>(self, other: I) -> bool
Iterator are lexicographically
less than those of another. Read more1.5.0 (const: unstable) · Sourcefn le<I>(self, other: I) -> bool
fn le<I>(self, other: I) -> bool
Iterator are lexicographically
less or equal to those of another. Read more1.5.0 (const: unstable) · Sourcefn gt<I>(self, other: I) -> bool
fn gt<I>(self, other: I) -> bool
Iterator are lexicographically
greater than those of another. Read more1.5.0 (const: unstable) · Sourcefn ge<I>(self, other: I) -> bool
fn ge<I>(self, other: I) -> bool
Iterator are lexicographically
greater than or equal to those of another. Read more